@@ -7,13 +7,9 @@ Simulation Environment for Atomistic and Molecular Modeling.
 
 import textwrap
 
-# Text handling
-from textwrap import dedent  # noqa: F401
-
 # Bring up the classes so that they appear to be directly in
 # the seamm package.
 
-from seamm.argument_parser import getParser  # noqa: F401
 from seamm.parameters import Parameter  # noqa: F401
 from seamm.parameters import Parameters  # noqa: F401
 from seamm.variables import Variables  # noqa: F401
@@ -44,9 +40,10 @@ wrap_stdout = textwrap.TextWrapper(width=120)
 
 # Handle versioneer
 from ._version import get_versions  # noqa: E402
+
 __author__ = """Paul Saxe"""
-__email__ = 'psaxe@molssi.org'
+__email__ = "psaxe@molssi.org"
 versions = get_versions()
-__version__ = versions['version']
-__git_revision__ = versions['full-revisionid']
+__version__ = versions["version"]
+__git_revision__ = versions["full-revisionid"]
 del get_versions, versions
